// Library     : DPMI
// Description : DOS Protected Mode Interface function library
// Module      : DPMI.H
// Version     : 1.00 - 01/09/1994
// Written by  : David A. Mair
// -----------------------------------------------------------------------------
// Change summary --------------------------------------------------------------
// -----------------------------------------------------------------------------
//    Date   | Version | Comment and author
// -----------------------------------------------------------------------------
//           |         |
//           |         |
// -----------------------------------------------------------------------------


#ifndef __DPMI_H__
#define __DPMI_H__


#include <windows.h>


#ifdef __cplusplus
extern "C" {
#endif


// Type definitions
typedef unsigned short SEGMENT;

typedef unsigned short SELECTOR;

typedef unsigned short NEAR_POINTER_16;

typedef unsigned long NEAR_POINTER_32;

typedef struct tagRealModeFarPointer
{
	WORD Offset;
	SEGMENT Segment;
} REAL_MODE_FAR_POINTER;

typedef struct tagProtectedMode16BitFarPointer
{
	WORD Offset;
	SELECTOR Selector;
} PROT_MODE_FAR_POINTER_16;

typedef struct tagProtectedMode32BitFarPointer
{
	unsigned long Offset;
	SELECTOR Selector;
} PROT_MODE_FAR_POINTER_32;

#pragma warn -bbf
typedef struct tagDescriptor
{
	WORD Limit0to15;
	WORD BaseAddress0to15;
	BYTE BaseAddress16to23;
	union
	{
		struct
		{
			BYTE Accessed : 1;
			BYTE Writable : 1;
			BYTE ExpandDown : 1;
			BYTE EqualsTwo : 2;
			BYTE DPL : 2;
			BYTE Present : 1;
		} DataSegment;
		struct
		{
			BYTE Accessed : 1;
			BYTE Readable : 1;
			BYTE Conforming : 1;
			BYTE EqualsThree : 2;
			BYTE DPL : 2;
			BYTE Present : 1;
		} CodeSegment;
		struct
		{
			BYTE Type : 4;
			BYTE EqualsZero : 1;
			BYTE DPL : 2;
			BYTE Present : 1;
		} SystemSegment;
	} SegmentType;
	WORD Limit16to20 : 4;
	WORD Available : 1;
	WORD : 1;
	WORD OperandSize : 1;
	WORD Granularity : 1;
	BYTE BaseAddress24to31;
} DESCRIPTOR;

typedef struct tagDescriptorList
{
	SELECTOR Selector;
	DESCRIPTOR Descriptor;
} DESCRIPTOR_LIST;

typedef struct tagAccessRights
{
	BYTE Accessed : 1;
	BYTE ReadWrite : 1;
	BYTE ExpandDown : 1;
	BYTE CodeSegment : 1;
	BYTE MustBeOne : 1;
	BYTE DPL : 2;
	BYTE Present : 1;
} ACCESS_RIGHTS;

typedef struct tag386Rights
{
	BYTE Reserved : 4;
	BYTE AVL : 1;
	BYTE MustBeZero : 1;
	BYTE Default32Bit : 1;
	BYTE PageGranular : 1;
} EXTENDED_ACCESS_RIGHTS;

typedef struct tagRealModeRegisters
{
	unsigned long edi;
	unsigned long esi;
	unsigned long ebp;
	unsigned long reserved;
	unsigned long ebx;
	unsigned long edx;
	unsigned long ecx;
	unsigned long eax;
	unsigned int flags;
	unsigned int es;
	unsigned int ds;
	unsigned int fs;
	unsigned int gs;
	unsigned int ip;
	unsigned int cs;
	unsigned int sp;
	unsigned int ss;
} REAL_MODE_REGS;

typedef struct tagDPMIVERSIONINFO
{
	BYTE MajorVersion;
	BYTE MinorVersion;
	WORD Host32Bit : 1;
	WORD ReflectedInterruptsToRealMode : 1;
	WORD VirtualMemorySupported : 1;
	WORD : 13;
	BYTE CPU;
	BYTE MasterPICBaseInt;
	BYTE SlavePICBaseInt;
} DPMI_VERSION_INFO;

typedef struct tagDPMICAPABILITIES
{
	WORD PageAccessedDirty : 1;
	WORD ExceptionsRestartable : 1;
	WORD DeviceMapping : 1;
	WORD ConventionalMemoryMapping : 1;
	WORD DemandZeroFill : 1;
	WORD WriteProtectClient : 1;
	WORD WriteProtectHost : 1;
	WORD : 9;
	BYTE MajorVersion;
	BYTE MinorVersion;
	char HostVendor[126];
} DPMI_CAPABILITIES;
#pragma warn .bbf


// Manifest constant macros
#define DPMI_SUCCESS      0x0000
#define LDT_SELECTOR      0x0004
#define SELECTOR_RPL      0x0003
#define SELECTOR_TI       0x0004


// Function like macros
#define InProtectedMode (GetCPUMode() == 0)
#define InRealMode 		(GetCPUMode() != 0)


BOOL ReleaseTimeSlice(void);
BOOL GetCPUMode(void);


WORD AllocateSpecificLDTDescriptor(SELECTOR Selector);
WORD FreeDOSMemoryBlock(SELECTOR Selector);
WORD FreeLDTDescriptor(SELECTOR SelectorToFree);
DPMI_CAPABILITIES GetDPMICapabilities(void);
DPMI_VERSION_INFO GetDPMIVersion(void);
REAL_MODE_FAR_POINTER GetRealModeInterruptVector(BYTE byIntNum);
DWORD GetSegmentLimit(SELECTOR Selector);
WORD GetSelectorIncrementValue(void);
void SetRealModeInterruptVector(BYTE byIntNum, REAL_MODE_FAR_POINTER rmfpNewVector);
WORD SetSegmentBaseAddress(SELECTOR Selector, DWORD BaseAddress);
WORD SetSegmentLimit(SELECTOR Selector, DWORD Limit);
WORD SetDescriptorAccessRights(SELECTOR Selector, ACCESS_RIGHTS AccessRights, EXTENDED_ACCESS_RIGHTS ExtendedRights);
WORD SetDescriptor(SELECTOR Selector, DESCRIPTOR Descriptor);

#ifdef __DPMI32__
WORD AllocateDOSMemoryBlock(WORD wParagraphs, SEGMENT *Segment, SELECTOR *Selector, WORD *pwLargestBlock);
WORD AllocateLDTDescriptors(WORD wNumberToAllocate, SELECTOR *BaseSelector);
WORD AllocateRealModeCallbackAddress(void *pCallbackProc, REAL_MODE_REGS *pRMRegs, REAL_MODE_FAR_POINTER *pCallbackAddress);
WORD CreateAliasDescriptor(SELECTOR Source, SELECTOR *Alias);
WORD FreeRealModeCallbackAddress(REAL_MODE_FAR_POINTER *pCallbackAddress);
WORD GetDescriptor(SELECTOR Selector, DESCRIPTOR *pDescriptor);        
ACCESS_RIGHTS GetDescriptorAccessRights(SELECTOR Selector, EXTENDED_ACCESS_RIGHTS *byExtendedRights);
WORD GetExtendedProcessorRealModeExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_32 *pmfp32ExceptionVector);
WORD GetExtendedProcessorProtModeExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_32 *pmfp32ExceptionVector);
WORD GetMultipleDescriptors(WORD *wNumberOfDescriptors, DESCRIPTOR_LIST *fpDescriptorList);
WORD GetProcessorExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_32 *pmfp32ExceptionVector);
PROT_MODE_FAR_POINTER_32 GetProtectedModeInterruptVector(BYTE byIntNum);
void GetRawModeSwitchAddresses(REAL_MODE_FAR_POINTER *pRealToProtectedMode, PROT_MODE_FAR_POINTER_32 *pProtectedToRealMode);
WORD GetSaveRestoreAddresses(REAL_MODE_FAR_POINTER *pRealModeRoutine, PROT_MODE_FAR_POINTER_32 *pProtectedModeRoutine);
WORD GetSegmentBaseAddress(SELECTOR Selector, DWORD *BaseAddress);
WORD ResizeDOSMemoryBlock(SELECTOR Selector, WORD wNewParagraphs, WORD *pwLargestBlock);
WORD SegmentToDescriptor(SEGMENT RealModeSegment, SELECTOR *SelectorForSegment);
WORD SetExtendedProcessorRealModeExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_32 pmfp32NewVector);
WORD SetExtendedProcessorProtModeExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_32 pmfp32NewVector);
WORD SetMultipleDescriptors(WORD *wNumberOfDescriptors, DESCRIPTOR_LIST *fpDescriptorList);
WORD SetProcessorExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_32 pmfp32NewVector);
WORD SetProtectedModeInterruptVector(BYTE byIntNum, PROT_MODE_FAR_POINTER_32 pmfp32NewVector);
WORD SimulateRealModeFarCall(REAL_MODE_REGS *pRMRegs, void *pStackData, WORD wStackDataWords);
WORD SimulateRealModeFarCallToInterrupt(REAL_MODE_REGS *pRMRegs, void *pStackData, WORD wStackDataWords);
WORD SimulateRealModeInterrupt(BYTE byIntNum, REAL_MODE_REGS *pRMRegs, void *pStackData, WORD wStackDataWords);
#else
WORD AllocateDOSMemoryBlock(WORD wParagraphs, SEGMENT far *Segment, SELECTOR far *Selector, WORD far *pwLargestBlock);
WORD AllocateLDTDescriptors(WORD wNumberToAllocate, SELECTOR far *BaseSelector);
WORD AllocateRealModeCallbackAddress(void far *fpCallbackProc, REAL_MODE_REGS far *fpRMRegs, REAL_MODE_FAR_POINTER far *fpCallbackAddress);
WORD CreateAliasDescriptor(SELECTOR Source, SELECTOR far *Alias);
WORD FreeRealModeCallbackAddress(REAL_MODE_FAR_POINTER far *pCallbackAddress);
WORD GetDescriptor(SELECTOR Selector, DESCRIPTOR far *pDescriptor);
ACCESS_RIGHTS GetDescriptorAccessRights(SELECTOR Selector, EXTENDED_ACCESS_RIGHTS far *byExtendedRights);
WORD GetExtendedProcessorRealModeExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_16 far *pmfp16ExceptionVector);
WORD GetExtendedProcessorProtModeExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_16 far *pmfp16ExceptionVector);
WORD GetMultipleDescriptors(WORD far *wNumberOfDescriptors, DESCRIPTOR_LIST far *fpDescriptorList);
WORD GetProcessorExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_16 far *pmfp16ExceptionVector);
PROT_MODE_FAR_POINTER_16 GetProtectedModeInterruptVector(BYTE byIntNum);
void GetRawModeSwitchAddresses(REAL_MODE_FAR_POINTER far *fpRealToProtectedMode, PROT_MODE_FAR_POINTER_16 far *fpProtectedToRealMode);
WORD GetSaveRestoreAddresses(REAL_MODE_FAR_POINTER far *fpRealModeRoutine, PROT_MODE_FAR_POINTER_16 far *fpProtectedModeRoutine);
WORD GetSegmentBaseAddress(SELECTOR Selector, DWORD far *BaseAddress);
WORD ResizeDOSMemoryBlock(SELECTOR Selector, WORD wNewParagraphs, WORD far *fpwLargestBlock);
WORD SegmentToDescriptor(SEGMENT RealModeSegment, SELECTOR far *SelectorForSegment);
WORD SetExtendedProcessorRealModeExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_16 pmfp16NewVector);
WORD SetExtendedProcessorProtModeExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_16 pmfp16NewVector);
WORD SetMultipleDescriptors(WORD far *wNumberOfDescriptors, DESCRIPTOR_LIST far *fpDescriptorList);
WORD SetProcessorExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_16 pmfp16NewVector);
WORD SetProtectedModeInterruptVector(BYTE byIntNum, PROT_MODE_FAR_POINTER_16 pmfp16NewVector);
WORD SimulateRealModeFarCall(REAL_MODE_REGS far *fpRMRegs, void far *pStackData, WORD wStackDataWords);
WORD SimulateRealModeFarCallToInterrupt(REAL_MODE_REGS far *fpRMRegs, void far *fpStackData, WORD wStackDataWords);
WORD SimulateRealModeInterrupt(BYTE byIntNum, REAL_MODE_REGS far *fpRMRegs, void far *fpStackData, WORD wStackDataWords);
#endif


#ifdef __cplusplus
}
#endif


#endif // __DPMI_H__

