// Library     : DPMI
// Description : DOS Protected Mode Interface function library
// Module      : DPMI01.C
// Version     : 1.00 - 01/09/1994
// Written by  : David A. Mair
// -----------------------------------------------------------------------------
// Change summary --------------------------------------------------------------
// -----------------------------------------------------------------------------
//    Date   | Version | Comment and author
// -----------------------------------------------------------------------------
//           |         |
//           |         |
// -----------------------------------------------------------------------------


// Inline assembler is included in this module
#pragma inline

#include "dpmi.h"


#ifdef __DPMI32__
WORD AllocateDOSMemoryBlock(WORD wParagraphs, SEGMENT *pSegment, SELECTOR *pSelector, WORD *pwLargestBlock)
#else
WORD AllocateDOSMemoryBlock(WORD wParagraphs, SEGMENT far *fpSegment, SELECTOR far *fpSelector, WORD far *fpwLargestBloc)
#endif
{
	WORD wRetVal;

	asm {
		mov		ax, 0x0100
		mov		bx, word ptr [wParagraphs]
		int		0x31
		push	es
		jc		admbSetRetVal

#ifdef __DPMI32__
		mov		ebx, dword ptr [pSegment]
		mov		word ptr [ebx], ax

		mov		ebx, dword ptr [pSelector]
		mov		word ptr [ebx], dx
#else
		les		bx, fpSegment
		mov		word ptr es:[bx], ax
		
		les		bx, fpSelector
		mov		word ptr es:[bx], dx
#endif
		mov		ax, 0x0000
		mov		bx, 0x0000
	}

admbSetRetVal:
	asm {
#ifdef __DPMI32__
		mov		edx, dword ptr [pwLargestBlock]
		mov		word ptr [edx], bx
#else
		mov		dx, bx
		les		bx, fpwLargestBlock
		mov		word ptr es:[bx], dx
#endif
		mov		word ptr [wRetVal], ax
		pop		es
	}
	
	return wRetVal;
}


WORD FreeDOSMemoryBlock(SELECTOR Selector)
{
	WORD wRetVal;

	asm {
		mov		ax, 0x0101
		mov		dx, word ptr [Selector]
		int		0x31
		
		jc		fdmbSetRetVal
		
		mov		ax, 0x0000
	}

fdmbSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}


#ifdef __DPMI32__
WORD ResizeDOSMemoryBlock(SELECTOR Selector, WORD wNewParagraphs, WORD *pwLargestBlock)
#else
WORD ResizeDOSMemoryBlock(SELECTOR Selector, WORD wNewParagraphs, WORD far *fpwLargestBlock)
#endif
{
	WORD wRetVal;

	asm {
		mov		ax, 0x0102
		mov		bx, word ptr [wNewParagraphs]
		mov		dx, word ptr [Selector]
		int		0x31
		
		jc		rdmbSetRetVal
		
		mov		ax, 0x0000
		mov		bx, 0x0000
	}

rdmbSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
#ifdef __DPMI32__
		mov		eax, dword ptr [pwLargestBlock]
		mov		word ptr [eax], bx
#else
		mov		ax, bx
		push	es
		les		bx, fpwLargestBlock
		mov		word ptr es:[bx], ax
		pop		es
#endif
	}

	return wRetVal;
}

